/*
  Tool            CodePageConv
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows GUI TSE
  Version         v1.0.1   2 Jan 2025

  If your Windows display language has accented characters, and you open or
  capture the output of a Windows command line command in Windows GUI TSE,
  then the accented characters are garbled.

  Example of such languages are Espaol, Franais, Portugus, and the Nordic
  languages.

  Executing this GUI TSE tool after opening or capturing the output can restore
  the accented characters.

  It does so by converting the external code page 437, 850 or 65001
  to GUI TSE's code page 1252 ("ANSI").

  CodePageConv is a tool: It can be executed manually after a GUI TSE input
  pipe or File Open, or from your own macro after a Dos(), lDos(), StartPgm(),
  or Capture() statement.

  Code pages are explained here:
    https://ecarlo.nl/tse/files/CodePages.html

  Caveat 1:
    Confusingly a lot of commands ignore the local code page and only use the
    global one, while other commands do adhere to a locally set code page.
    Simply not setting a local code page avoids that problem.

  Caveat 2:
    All of the above assumes, that GUI TSE uses the Courier New font or another
    ANSI-compatible font. E.g. "Terminal" is not an ANSI-compatibel font.


  LIMITATIONS

  If a character from code page 437, 850 or 65001 does not have a code page
  1252 equivalent, then the tool will convert the character to a question mark.
  This typically is the case for (box) drawing characters, like lines.
  Fortunately these are typically not used by Windows command prompt commands.

  This is a simplistic tool for handling the output of typical Windows
  commands, which use a limited character set.
  This tool is not usable for handling the full Unicode character set.
  For that I advise the use of TSE's Unicode extension:
    https://ecarlo.nl/tse/ToolsAndExtensions.html#Unicode


  INSTALLATION

  Copy this file to TSE's "mac" directory and compile it there.


  USAGE

  Execute this tool as a macro from anywhere.

  A typical use case is to execute this macro from your own macro, just after
  your own macro used Dos(), lDos(), StartPgm() or Capture() statement.

  Another use case is to use this macro manually after executing a command from
  the "cmd" prompt and editing its output.
  This also works when piping output to TSE, and even like this:
    ipconfig | g32.exe -eCodePageConv

  If a known local code page is set, then you can overrule CodePageConv using
  the global code page by explicitly passing the local code page number in the
  macro command line.


  HISTORY

  v1       2 Jan 2025
    Initial release.

  v1.0.1   2 Jan 2025
    Simplified the documentation and rearranged some code.

*/





// Start of compatibility restrictions and mitigations ...

#ifdef LINUX
  This macro is not compatible with LINUX TSE.
#endif

#ifndef WIN32
   16-bit versions of TSE are not supported. You need at least TSE 4.4.
#endif

#ifndef EDITOR_VERSION
   Editor Version is older than TSE 3.0. You need at least TSE 4.4.
#endif

#if EDITOR_VERSION < 4400h
   Editor Version is older than TSE 4.4. You need at least TSE 4.4.
#endif

// End of compatibility restrictions and mitigations.





// Global constants and semi-constants
string MACRO_NAME [MAXSTRINGLEN] = ''





// Global variables
string  cp_437_conv_table [MAXSTRINGLEN] = ''
string  cp_850_conv_table [MAXSTRINGLEN] = ''
string  cp_conv_table     [MAXSTRINGLEN] = ''
integer macro_id                         = 0


proc init_cp_437()
  integer i = 0

  for i = 1 to 127
    cp_437_conv_table[i] = Chr(i)
  endfor

  cp_437_conv_table[128] = Chr(199) // ''
  cp_437_conv_table[129] = Chr(252) // ''
  cp_437_conv_table[130] = Chr(233) // ''
  cp_437_conv_table[131] = Chr(226) // ''
  cp_437_conv_table[132] = Chr(228) // ''
  cp_437_conv_table[133] = Chr(224) // ''
  cp_437_conv_table[134] = Chr(229) // ''
  cp_437_conv_table[135] = Chr(231) // ''
  cp_437_conv_table[136] = Chr(234) // ''
  cp_437_conv_table[137] = Chr(235) // ''
  cp_437_conv_table[138] = Chr(232) // ''
  cp_437_conv_table[139] = Chr(239) // ''
  cp_437_conv_table[140] = Chr(238) // ''
  cp_437_conv_table[141] = Chr(236) // ''
  cp_437_conv_table[142] = Chr(196) // ''
  cp_437_conv_table[143] = Chr(197) // ''
  cp_437_conv_table[144] = Chr(201) // ''
  cp_437_conv_table[145] = Chr(230) // ''
  cp_437_conv_table[146] = Chr(198) // ''
  cp_437_conv_table[147] = Chr(244) // ''
  cp_437_conv_table[148] = Chr(246) // ''
  cp_437_conv_table[149] = Chr(242) // ''
  cp_437_conv_table[150] = Chr(251) // ''
  cp_437_conv_table[151] = Chr(249) // ''
  cp_437_conv_table[152] = Chr(255) // ''
  cp_437_conv_table[153] = Chr(214) // ''
  cp_437_conv_table[154] = Chr(220) // ''
  cp_437_conv_table[155] = Chr(162) // ''
  cp_437_conv_table[156] = Chr(163) // ''
  cp_437_conv_table[157] = Chr(165) // ''
  cp_437_conv_table[158] = Chr( 63) // '?'
  cp_437_conv_table[159] = Chr(131) // ''
  cp_437_conv_table[160] = Chr(225) // ''
  cp_437_conv_table[161] = Chr(237) // ''
  cp_437_conv_table[162] = Chr(243) // ''
  cp_437_conv_table[163] = Chr(250) // ''
  cp_437_conv_table[164] = Chr(241) // ''
  cp_437_conv_table[165] = Chr(209) // ''
  cp_437_conv_table[166] = Chr(170) // ''
  cp_437_conv_table[167] = Chr(186) // ''
  cp_437_conv_table[168] = Chr(191) // ''
  cp_437_conv_table[169] = Chr( 63) // '?'
  cp_437_conv_table[170] = Chr(172) // ''
  cp_437_conv_table[171] = Chr(189) // ''
  cp_437_conv_table[172] = Chr(188) // ''
  cp_437_conv_table[173] = Chr(161) // ''
  cp_437_conv_table[174] = Chr(171) // ''
  cp_437_conv_table[175] = Chr(187) // ''
  cp_437_conv_table[176] = Chr( 63) // '?'
  cp_437_conv_table[177] = Chr( 63) // '?'
  cp_437_conv_table[178] = Chr( 63) // '?'
  cp_437_conv_table[179] = Chr( 63) // '?'
  cp_437_conv_table[180] = Chr( 63) // '?'
  cp_437_conv_table[181] = Chr( 63) // '?'
  cp_437_conv_table[182] = Chr( 63) // '?'
  cp_437_conv_table[183] = Chr( 63) // '?'
  cp_437_conv_table[184] = Chr( 63) // '?'
  cp_437_conv_table[185] = Chr( 63) // '?'
  cp_437_conv_table[186] = Chr( 63) // '?'
  cp_437_conv_table[187] = Chr( 63) // '?'
  cp_437_conv_table[188] = Chr( 63) // '?'
  cp_437_conv_table[189] = Chr( 63) // '?'
  cp_437_conv_table[190] = Chr( 63) // '?'
  cp_437_conv_table[191] = Chr( 63) // '?'
  cp_437_conv_table[192] = Chr( 63) // '?'
  cp_437_conv_table[193] = Chr( 63) // '?'
  cp_437_conv_table[194] = Chr( 63) // '?'
  cp_437_conv_table[195] = Chr( 63) // '?'
  cp_437_conv_table[196] = Chr( 63) // '?'
  cp_437_conv_table[197] = Chr( 63) // '?'
  cp_437_conv_table[198] = Chr( 63) // '?'
  cp_437_conv_table[199] = Chr( 63) // '?'
  cp_437_conv_table[200] = Chr( 63) // '?'
  cp_437_conv_table[201] = Chr( 63) // '?'
  cp_437_conv_table[202] = Chr( 63) // '?'
  cp_437_conv_table[203] = Chr( 63) // '?'
  cp_437_conv_table[204] = Chr( 63) // '?'
  cp_437_conv_table[205] = Chr( 63) // '?'
  cp_437_conv_table[206] = Chr( 63) // '?'
  cp_437_conv_table[207] = Chr( 63) // '?'
  cp_437_conv_table[208] = Chr( 63) // '?'
  cp_437_conv_table[209] = Chr( 63) // '?'
  cp_437_conv_table[210] = Chr( 63) // '?'
  cp_437_conv_table[211] = Chr( 63) // '?'
  cp_437_conv_table[212] = Chr( 63) // '?'
  cp_437_conv_table[213] = Chr( 63) // '?'
  cp_437_conv_table[214] = Chr( 63) // '?'
  cp_437_conv_table[215] = Chr( 63) // '?'
  cp_437_conv_table[216] = Chr( 63) // '?'
  cp_437_conv_table[217] = Chr( 63) // '?'
  cp_437_conv_table[218] = Chr( 63) // '?'
  cp_437_conv_table[219] = Chr( 63) // '?'
  cp_437_conv_table[220] = Chr( 63) // '?'
  cp_437_conv_table[221] = Chr( 63) // '?'
  cp_437_conv_table[222] = Chr( 63) // '?'
  cp_437_conv_table[223] = Chr( 63) // '?'
  cp_437_conv_table[224] = Chr( 63) // '?'
  cp_437_conv_table[225] = Chr(223) // ''
  cp_437_conv_table[226] = Chr( 63) // '?'
  cp_437_conv_table[227] = Chr( 63) // '?'
  cp_437_conv_table[228] = Chr( 63) // '?'
  cp_437_conv_table[229] = Chr( 63) // '?'
  cp_437_conv_table[230] = Chr(181) // ''
  cp_437_conv_table[231] = Chr( 63) // '?'
  cp_437_conv_table[232] = Chr( 63) // '?'
  cp_437_conv_table[233] = Chr( 63) // '?'
  cp_437_conv_table[234] = Chr( 63) // '?'
  cp_437_conv_table[235] = Chr( 63) // '?'
  cp_437_conv_table[236] = Chr( 63) // '?'
  cp_437_conv_table[237] = Chr( 63) // '?'
  cp_437_conv_table[238] = Chr( 63) // '?'
  cp_437_conv_table[239] = Chr( 63) // '?'
  cp_437_conv_table[240] = Chr( 63) // '?'
  cp_437_conv_table[241] = Chr(177) // ''
  cp_437_conv_table[242] = Chr( 63) // '?'
  cp_437_conv_table[243] = Chr( 63) // '?'
  cp_437_conv_table[244] = Chr( 63) // '?'
  cp_437_conv_table[245] = Chr( 63) // '?'
  cp_437_conv_table[246] = Chr(247) // ''
  cp_437_conv_table[247] = Chr( 63) // '?'
  cp_437_conv_table[248] = Chr(176) // ''
  cp_437_conv_table[249] = Chr( 63) // '?'
  cp_437_conv_table[250] = Chr(183) // ''
  cp_437_conv_table[251] = Chr( 63) // '?'
  cp_437_conv_table[252] = Chr( 63) // '?'
  cp_437_conv_table[253] = Chr(178) // ''
  cp_437_conv_table[254] = Chr( 63) // '?'
  cp_437_conv_table[255] = Chr(160) // NBSP
end init_cp_437



proc init_cp_850()
  integer i = 0

  for i = 1 to 127
    cp_850_conv_table[i] = Chr(i)
  endfor

  cp_850_conv_table[128] = Chr(199) // ''
  cp_850_conv_table[129] = Chr(252) // ''
  cp_850_conv_table[130] = Chr(233) // ''
  cp_850_conv_table[131] = Chr(226) // ''
  cp_850_conv_table[132] = Chr(228) // ''
  cp_850_conv_table[133] = Chr(224) // ''
  cp_850_conv_table[134] = Chr(229) // ''
  cp_850_conv_table[135] = Chr(231) // ''
  cp_850_conv_table[136] = Chr(234) // ''
  cp_850_conv_table[137] = Chr(235) // ''
  cp_850_conv_table[138] = Chr(232) // ''
  cp_850_conv_table[139] = Chr(239) // ''
  cp_850_conv_table[140] = Chr(238) // ''
  cp_850_conv_table[141] = Chr(236) // ''
  cp_850_conv_table[142] = Chr(196) // ''
  cp_850_conv_table[143] = Chr(197) // ''
  cp_850_conv_table[144] = Chr(201) // ''
  cp_850_conv_table[145] = Chr(230) // ''
  cp_850_conv_table[146] = Chr(198) // ''
  cp_850_conv_table[147] = Chr(244) // ''
  cp_850_conv_table[148] = Chr(246) // ''
  cp_850_conv_table[149] = Chr(242) // ''
  cp_850_conv_table[150] = Chr(251) // ''
  cp_850_conv_table[151] = Chr(249) // ''
  cp_850_conv_table[152] = Chr(255) // ''
  cp_850_conv_table[153] = Chr(214) // ''
  cp_850_conv_table[154] = Chr(220) // ''
  cp_850_conv_table[155] = Chr(248) // ''
  cp_850_conv_table[156] = Chr(163) // ''
  cp_850_conv_table[157] = Chr(216) // ''
  cp_850_conv_table[158] = Chr(215) // ''
  cp_850_conv_table[159] = Chr(131) // ''
  cp_850_conv_table[160] = Chr(225) // ''
  cp_850_conv_table[161] = Chr(237) // ''
  cp_850_conv_table[162] = Chr(243) // ''
  cp_850_conv_table[163] = Chr(250) // ''
  cp_850_conv_table[164] = Chr(241) // ''
  cp_850_conv_table[165] = Chr(209) // ''
  cp_850_conv_table[166] = Chr(170) // ''
  cp_850_conv_table[167] = Chr(186) // ''
  cp_850_conv_table[168] = Chr(191) // ''
  cp_850_conv_table[169] = Chr(174) // ''
  cp_850_conv_table[170] = Chr(172) // ''
  cp_850_conv_table[171] = Chr(189) // ''
  cp_850_conv_table[172] = Chr(188) // ''
  cp_850_conv_table[173] = Chr(161) // ''
  cp_850_conv_table[174] = Chr(171) // ''
  cp_850_conv_table[175] = Chr(187) // ''
  cp_850_conv_table[176] = Chr( 63) // '?'
  cp_850_conv_table[177] = Chr( 63) // '?'
  cp_850_conv_table[178] = Chr( 63) // '?'
  cp_850_conv_table[179] = Chr( 63) // '?'
  cp_850_conv_table[180] = Chr( 63) // '?'
  cp_850_conv_table[181] = Chr(193) // ''
  cp_850_conv_table[182] = Chr(194) // ''
  cp_850_conv_table[183] = Chr(192) // ''
  cp_850_conv_table[184] = Chr(169) // ''
  cp_850_conv_table[185] = Chr( 63) // '?'
  cp_850_conv_table[186] = Chr( 63) // '?'
  cp_850_conv_table[187] = Chr( 63) // '?'
  cp_850_conv_table[188] = Chr( 63) // '?'
  cp_850_conv_table[189] = Chr(162) // ''
  cp_850_conv_table[190] = Chr(165) // ''
  cp_850_conv_table[191] = Chr( 63) // '?'
  cp_850_conv_table[192] = Chr( 63) // '?'
  cp_850_conv_table[193] = Chr( 63) // '?'
  cp_850_conv_table[194] = Chr( 63) // '?'
  cp_850_conv_table[195] = Chr( 63) // '?'
  cp_850_conv_table[196] = Chr( 63) // '?'
  cp_850_conv_table[197] = Chr( 63) // '?'
  cp_850_conv_table[198] = Chr(227) // ''
  cp_850_conv_table[199] = Chr(195) // ''
  cp_850_conv_table[200] = Chr( 63) // '?'
  cp_850_conv_table[201] = Chr( 63) // '?'
  cp_850_conv_table[202] = Chr( 63) // '?'
  cp_850_conv_table[203] = Chr( 63) // '?'
  cp_850_conv_table[204] = Chr( 63) // '?'
  cp_850_conv_table[205] = Chr( 63) // '?'
  cp_850_conv_table[206] = Chr( 63) // '?'
  cp_850_conv_table[207] = Chr(164) // ''
  cp_850_conv_table[208] = Chr(240) // ''
  cp_850_conv_table[209] = Chr(208) // ''
  cp_850_conv_table[210] = Chr(202) // ''
  cp_850_conv_table[211] = Chr(203) // ''
  cp_850_conv_table[212] = Chr(200) // ''
  cp_850_conv_table[213] = Chr( 63) // '?'
  cp_850_conv_table[214] = Chr(205) // ''
  cp_850_conv_table[215] = Chr(206) // ''
  cp_850_conv_table[216] = Chr(207) // ''
  cp_850_conv_table[217] = Chr( 63) // '?'
  cp_850_conv_table[218] = Chr( 63) // '?'
  cp_850_conv_table[219] = Chr( 63) // '?'
  cp_850_conv_table[220] = Chr( 63) // '?'
  cp_850_conv_table[221] = Chr(166) // ''
  cp_850_conv_table[222] = Chr(204) // ''
  cp_850_conv_table[223] = Chr( 63) // '?'
  cp_850_conv_table[224] = Chr(211) // ''
  cp_850_conv_table[225] = Chr(223) // ''
  cp_850_conv_table[226] = Chr(212) // ''
  cp_850_conv_table[227] = Chr(210) // ''
  cp_850_conv_table[228] = Chr(245) // ''
  cp_850_conv_table[229] = Chr(213) // ''
  cp_850_conv_table[230] = Chr(181) // ''
  cp_850_conv_table[231] = Chr(254) // ''
  cp_850_conv_table[232] = Chr(222) // ''
  cp_850_conv_table[233] = Chr(218) // ''
  cp_850_conv_table[234] = Chr(219) // ''
  cp_850_conv_table[235] = Chr(217) // ''
  cp_850_conv_table[236] = Chr(253) // ''
  cp_850_conv_table[237] = Chr(221) // ''
  cp_850_conv_table[238] = Chr(175) // ''
  cp_850_conv_table[239] = Chr(180) // ''
  cp_850_conv_table[240] = Chr(173) // SHY
  cp_850_conv_table[241] = Chr(177) // ''
  cp_850_conv_table[242] = Chr( 63) // '?'
  cp_850_conv_table[243] = Chr(190) // ''
  cp_850_conv_table[244] = Chr(182) // ''
  cp_850_conv_table[245] = Chr(167) // ''
  cp_850_conv_table[246] = Chr(247) // ''
  cp_850_conv_table[247] = Chr(184) // ''
  cp_850_conv_table[248] = Chr(176) // ''
  cp_850_conv_table[249] = Chr(168) // ''
  cp_850_conv_table[250] = Chr(183) // ''
  cp_850_conv_table[251] = Chr(185) // ''
  cp_850_conv_table[252] = Chr(179) // ''
  cp_850_conv_table[253] = Chr(178) // ''
  cp_850_conv_table[254] = Chr( 63) // '?'
  cp_850_conv_table[255] = Chr(160) // NBSP
end init_cp_850


proc init_cp_65001()
  PushLocation()
  macro_id = CreateTempBuffer()
  ChangeCurrFilename(MACRO_NAME + ':ConversionTable',
                     _DONT_PROMPT_|_DONT_EXPAND_|_OVERWRITE_)

  AddLine(Chr(226) + Chr(130)+ Chr(172))
  AddLine(Chr(194) + Chr(129))
  AddLine(Chr(226) + Chr(128)+ Chr(154))
  AddLine(Chr(198) + Chr(146))
  AddLine(Chr(226) + Chr(128)+ Chr(158))
  AddLine(Chr(226) + Chr(128)+ Chr(166))
  AddLine(Chr(226) + Chr(128)+ Chr(160))
  AddLine(Chr(226) + Chr(128)+ Chr(161))
  AddLine(Chr(203) + Chr(134))
  AddLine(Chr(226) + Chr(128)+ Chr(176))
  AddLine(Chr(197) + Chr(160))
  AddLine(Chr(226) + Chr(128)+ Chr(185))
  AddLine(Chr(197) + Chr(146))
  AddLine(Chr(194) + Chr(141))
  AddLine(Chr(197) + Chr(189))
  AddLine(Chr(194) + Chr(143))
  AddLine(Chr(194) + Chr(144))
  AddLine(Chr(226) + Chr(128)+ Chr(152))
  AddLine(Chr(226) + Chr(128)+ Chr(153))
  AddLine(Chr(226) + Chr(128)+ Chr(156))
  AddLine(Chr(226) + Chr(128)+ Chr(157))
  AddLine(Chr(226) + Chr(128)+ Chr(162))
  AddLine(Chr(226) + Chr(128)+ Chr(147))
  AddLine(Chr(226) + Chr(128)+ Chr(148))
  AddLine(Chr(203) + Chr(156))
  AddLine(Chr(226) + Chr(132)+ Chr(162))
  AddLine(Chr(197) + Chr(161))
  AddLine(Chr(226) + Chr(128)+ Chr(186))
  AddLine(Chr(197) + Chr(147))
  AddLine(Chr(194) + Chr(157))
  AddLine(Chr(197) + Chr(190))
  AddLine(Chr(197) + Chr(184))
  AddLine(Chr(194) + Chr(160))
  AddLine(Chr(194) + Chr(161))
  AddLine(Chr(194) + Chr(162))
  AddLine(Chr(194) + Chr(163))
  AddLine(Chr(194) + Chr(164))
  AddLine(Chr(194) + Chr(165))
  AddLine(Chr(194) + Chr(166))
  AddLine(Chr(194) + Chr(167))
  AddLine(Chr(194) + Chr(168))
  AddLine(Chr(194) + Chr(169))
  AddLine(Chr(194) + Chr(170))
  AddLine(Chr(194) + Chr(171))
  AddLine(Chr(194) + Chr(172))
  AddLine(Chr(194) + Chr(173))
  AddLine(Chr(194) + Chr(174))
  AddLine(Chr(194) + Chr(175))
  AddLine(Chr(194) + Chr(176))
  AddLine(Chr(194) + Chr(177))
  AddLine(Chr(194) + Chr(178))
  AddLine(Chr(194) + Chr(179))
  AddLine(Chr(194) + Chr(180))
  AddLine(Chr(194) + Chr(181))
  AddLine(Chr(194) + Chr(182))
  AddLine(Chr(194) + Chr(183))
  AddLine(Chr(194) + Chr(184))
  AddLine(Chr(194) + Chr(185))
  AddLine(Chr(194) + Chr(186))
  AddLine(Chr(194) + Chr(187))
  AddLine(Chr(194) + Chr(188))
  AddLine(Chr(194) + Chr(189))
  AddLine(Chr(194) + Chr(190))
  AddLine(Chr(194) + Chr(191))
  AddLine(Chr(195) + Chr(128))
  AddLine(Chr(195) + Chr(129))
  AddLine(Chr(195) + Chr(130))
  AddLine(Chr(195) + Chr(131))
  AddLine(Chr(195) + Chr(132))
  AddLine(Chr(195) + Chr(133))
  AddLine(Chr(195) + Chr(134))
  AddLine(Chr(195) + Chr(135))
  AddLine(Chr(195) + Chr(136))
  AddLine(Chr(195) + Chr(137))
  AddLine(Chr(195) + Chr(138))
  AddLine(Chr(195) + Chr(139))
  AddLine(Chr(195) + Chr(140))
  AddLine(Chr(195) + Chr(141))
  AddLine(Chr(195) + Chr(142))
  AddLine(Chr(195) + Chr(143))
  AddLine(Chr(195) + Chr(144))
  AddLine(Chr(195) + Chr(145))
  AddLine(Chr(195) + Chr(146))
  AddLine(Chr(195) + Chr(147))
  AddLine(Chr(195) + Chr(148))
  AddLine(Chr(195) + Chr(149))
  AddLine(Chr(195) + Chr(150))
  AddLine(Chr(195) + Chr(151))
  AddLine(Chr(195) + Chr(152))
  AddLine(Chr(195) + Chr(153))
  AddLine(Chr(195) + Chr(154))
  AddLine(Chr(195) + Chr(155))
  AddLine(Chr(195) + Chr(156))
  AddLine(Chr(195) + Chr(157))
  AddLine(Chr(195) + Chr(158))
  AddLine(Chr(195) + Chr(159))
  AddLine(Chr(195) + Chr(160))
  AddLine(Chr(195) + Chr(161))
  AddLine(Chr(195) + Chr(162))
  AddLine(Chr(195) + Chr(163))
  AddLine(Chr(195) + Chr(164))
  AddLine(Chr(195) + Chr(165))
  AddLine(Chr(195) + Chr(166))
  AddLine(Chr(195) + Chr(167))
  AddLine(Chr(195) + Chr(168))
  AddLine(Chr(195) + Chr(169))
  AddLine(Chr(195) + Chr(170))
  AddLine(Chr(195) + Chr(171))
  AddLine(Chr(195) + Chr(172))
  AddLine(Chr(195) + Chr(173))
  AddLine(Chr(195) + Chr(174))
  AddLine(Chr(195) + Chr(175))
  AddLine(Chr(195) + Chr(176))
  AddLine(Chr(195) + Chr(177))
  AddLine(Chr(195) + Chr(178))
  AddLine(Chr(195) + Chr(179))
  AddLine(Chr(195) + Chr(180))
  AddLine(Chr(195) + Chr(181))
  AddLine(Chr(195) + Chr(182))
  AddLine(Chr(195) + Chr(183))
  AddLine(Chr(195) + Chr(184))
  AddLine(Chr(195) + Chr(185))
  AddLine(Chr(195) + Chr(186))
  AddLine(Chr(195) + Chr(187))
  AddLine(Chr(195) + Chr(188))
  AddLine(Chr(195) + Chr(189))
  AddLine(Chr(195) + Chr(190))
  AddLine(Chr(195) + Chr(191))

  PopLocation()
end init_cp_65001


proc conv_65001_encoding()
  string  cp1252_byte [1] = ''
  integer old_Insert      = Set(Insert, ON)
  integer org_id          = GetBufferId()
  string  utf8_bytes  [6] = ''
  integer utf8_length     = 0
  PushPosition()
  BegFile()
  while lFind('[\d128-\d255]', 'x')
    utf8_length = Pos('0', Str(CurrChar(), 2)) - 1
    if (utf8_length in 2 .. 6)
      utf8_bytes = GetText(CurrPos(), utf8_length)
      GotoBufferId(macro_id)
      if lFind(utf8_bytes, '^g$')
        cp1252_byte = Chr(CurrLine() + 127)
      else
        cp1252_byte = '?'
      endif
      GotoBufferId(org_id)
    else
      cp1252_byte = '?'
    endif
    DelChar(utf8_length)
    InsertText(cp1252_byte)
  endwhile
  PopPosition()
  Set(Insert, old_Insert)
end conv_65001_encoding


proc conv_1byte_encoding()
  integer old_Insert        = Set(Insert, OFF)
  integer curr_char_asc     = 0
  string  curr_char_chr [1] = ''
  PushPosition()
  BegFile()
  while lFind('[\d128-\d255]', 'x')
    curr_char_asc = CurrChar()
    curr_char_chr = cp_conv_table[curr_char_asc]
    InsertText(curr_char_chr)
  endwhile
  PopPosition()
  Set(Insert, old_Insert)
end conv_1byte_encoding


proc WhenPurged()
  AbandonFile(macro_id)
end WhenPurged


proc WhenLoaded()
  MACRO_NAME = SplitPath(CurrMacroFilename(), _NAME_)

  cp_437_conv_table = Format('': 255: '?')
  cp_850_conv_table = Format('': 255: '?')
end WhenLoaded


proc Main()
  integer code_page = 0

  if isGUI()
    code_page = Val(Query(MacroCmdLine))

    if not code_page
      code_page = Query(CodePage)
    endif

    case code_page
      when 437
        init_cp_437()
        cp_conv_table = cp_437_conv_table
        conv_1byte_encoding()
      when 850
        init_cp_850()
        cp_conv_table = cp_850_conv_table
        conv_1byte_encoding()
      when 65001
        init_cp_65001()
        conv_65001_encoding()
      otherwise
        Warn('Converting from code page'; Query(CodePage); 'is not supported.')
    endcase
  else
    Warn('This tool only works in the Windows GUI variant of TSE.')
  endif
  PurgeMacro(CurrMacroFilename())
end Main

